package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.AbstractMethodDeclaration;
import org.eclipse.jdt.internal.compiler.ast.Annotation;
import org.eclipse.jdt.internal.compiler.ast.LocalDeclaration;
import org.eclipse.jdt.internal.compiler.ast.TypeReference;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.CatchParameterBinding;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.InvocationSite;
import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodScope;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;

public class Argument extends LocalDeclaration {
  private static final char[] SET = "set".toCharArray();
  
  public Argument(char[] name, long posNom, TypeReference tr, int modifiers) {
    super(name, (int)(posNom >>> 32L), (int)posNom);
    this.declarationSourceEnd = (int)posNom;
    this.modifiers = modifiers;
    this.type = tr;
    if (tr != null)
      this.bits |= tr.bits & 0x100000; 
    this.bits |= 0x40000004;
  }
  
  public Argument(char[] name, long posNom, TypeReference tr, int modifiers, boolean typeElided) {
    super(name, (int)(posNom >>> 32L), (int)posNom);
    this.declarationSourceEnd = (int)posNom;
    this.modifiers = modifiers;
    this.type = tr;
    if (tr != null)
      this.bits |= tr.bits & 0x100000; 
    this.bits |= 0x40000006;
  }
  
  public boolean isRecoveredFromLoneIdentifier() {
    return false;
  }
  
  public TypeBinding createBinding(MethodScope scope, TypeBinding typeBinding) {
    if (this.binding == null) {
      this.binding = new LocalVariableBinding(this, typeBinding, this.modifiers, scope);
    } else if (!this.binding.type.isValidBinding()) {
      AbstractMethodDeclaration methodDecl = scope.referenceMethod();
      if (methodDecl != null) {
        MethodBinding methodBinding = methodDecl.binding;
        if (methodBinding != null)
          methodBinding.tagBits |= 0x200L; 
      } 
    } 
    if ((this.binding.tagBits & 0x200000000L) == 0L) {
      resolveAnnotations((BlockScope)scope, this.annotations, (Binding)this.binding, true);
      if ((scope.compilerOptions()).sourceLevel >= 3407872L) {
        Annotation.isTypeUseCompatible(this.type, (Scope)scope, this.annotations);
        scope.validateNullAnnotation(this.binding.tagBits, this.type, this.annotations);
      } 
    } 
    this.binding.declaration = this;
    return this.binding.type;
  }
  
  public TypeBinding bind(MethodScope scope, TypeBinding typeBinding, boolean used) {
    TypeBinding newTypeBinding = createBinding(scope, typeBinding);
    Binding existingVariable = scope.getBinding(this.name, 3, (InvocationSite)this, false);
    if (existingVariable != null && existingVariable.isValidBinding()) {
      boolean localExists = existingVariable instanceof LocalVariableBinding;
      if (localExists && this.hiddenVariableDepth == 0) {
        if ((this.bits & 0x200000) != 0 && scope.isLambdaSubscope()) {
          scope.problemReporter().lambdaRedeclaresArgument(this);
        } else {
          scope.problemReporter().redefineArgument(this);
        } 
      } else {
        boolean isSpecialArgument = false;
        if (existingVariable instanceof org.eclipse.jdt.internal.compiler.lookup.FieldBinding)
          if (scope.isInsideConstructor()) {
            isSpecialArgument = true;
          } else {
            AbstractMethodDeclaration methodDecl = scope.referenceMethod();
            if (methodDecl != null && CharOperation.prefixEquals(SET, methodDecl.selector))
              isSpecialArgument = true; 
          }  
        scope.problemReporter().localVariableHiding(this, existingVariable, isSpecialArgument);
      } 
    } 
    scope.addLocalVariable(this.binding);
    this.binding.useFlag = used ? 1 : 0;
    return newTypeBinding;
  }
  
  public int getKind() {
    return ((this.bits & 0x4) != 0) ? 5 : 4;
  }
  
  public boolean isArgument() {
    return true;
  }
  
  public boolean isVarArgs() {
    return (this.type != null && (this.type.bits & 0x4000) != 0);
  }
  
  public boolean hasElidedType() {
    return ((this.bits & 0x2) != 0);
  }
  
  public boolean hasNullTypeAnnotation(TypeReference.AnnotationPosition position) {
    return !(!TypeReference.containsNullAnnotation(this.annotations) && (
      this.type == null || !this.type.hasNullTypeAnnotation(position)));
  }
  
  public StringBuffer print(int indent, StringBuffer output) {
    printIndent(indent, output);
    printModifiers(this.modifiers, output);
    if (this.annotations != null) {
      printAnnotations(this.annotations, output);
      output.append(' ');
    } 
    if (this.type == null) {
      output.append("<no type> ");
    } else {
      this.type.print(0, output).append(' ');
    } 
    return output.append(this.name);
  }
  
  public StringBuffer printStatement(int indent, StringBuffer output) {
    return print(indent, output).append(';');
  }
  
  public TypeBinding resolveForCatch(BlockScope scope) {
    boolean hasError;
    TypeBinding exceptionType = this.type.resolveType(scope, true);
    if (exceptionType == null) {
      hasError = true;
    } else {
      hasError = false;
      switch (exceptionType.kind()) {
        case 260:
          if (exceptionType.isBoundParameterizedType()) {
            hasError = true;
            scope.problemReporter().invalidParameterizedExceptionType(exceptionType, (ASTNode)this);
          } 
          break;
        case 4100:
          scope.problemReporter().invalidTypeVariableAsException(exceptionType, (ASTNode)this);
          hasError = true;
          break;
      } 
      if (exceptionType.findSuperTypeOriginatingFrom(21, true) == null && exceptionType.isValidBinding()) {
        scope.problemReporter().cannotThrowType((ASTNode)this.type, exceptionType);
        hasError = true;
      } 
    } 
    Binding existingVariable = scope.getBinding(this.name, 3, (InvocationSite)this, false);
    if (existingVariable != null && existingVariable.isValidBinding())
      if (existingVariable instanceof LocalVariableBinding && this.hiddenVariableDepth == 0) {
        scope.problemReporter().redefineArgument(this);
      } else {
        scope.problemReporter().localVariableHiding(this, existingVariable, false);
      }  
    if ((this.type.bits & 0x20000000) != 0) {
      this.binding = (LocalVariableBinding)new CatchParameterBinding(this, exceptionType, this.modifiers | 0x10, false);
      this.binding.tagBits |= 0x1000L;
    } else {
      this.binding = (LocalVariableBinding)new CatchParameterBinding(this, exceptionType, this.modifiers, false);
    } 
    resolveAnnotations(scope, this.annotations, (Binding)this.binding, true);
    Annotation.isTypeUseCompatible(this.type, (Scope)scope, this.annotations);
    if ((scope.compilerOptions()).isAnnotationBasedNullAnalysisEnabled && (
      this.type.hasNullTypeAnnotation(TypeReference.AnnotationPosition.ANY) || TypeReference.containsNullAnnotation(this.annotations)))
      scope.problemReporter().nullAnnotationUnsupportedLocation(this.type); 
    scope.addLocalVariable(this.binding);
    this.binding.setConstant(Constant.NotAConstant);
    if (hasError)
      return null; 
    return exceptionType;
  }
  
  public void traverse(ASTVisitor visitor, BlockScope scope) {
    if (visitor.visit(this, scope)) {
      if (this.annotations != null) {
        int annotationsLength = this.annotations.length;
        for (int i = 0; i < annotationsLength; i++)
          this.annotations[i].traverse(visitor, scope); 
      } 
      if (this.type != null)
        this.type.traverse(visitor, scope); 
    } 
    visitor.endVisit(this, scope);
  }
  
  public void traverse(ASTVisitor visitor, ClassScope scope) {
    if (visitor.visit(this, scope)) {
      if (this.annotations != null) {
        int annotationsLength = this.annotations.length;
        for (int i = 0; i < annotationsLength; i++)
          this.annotations[i].traverse(visitor, scope); 
      } 
      if (this.type != null)
        this.type.traverse(visitor, scope); 
    } 
    visitor.endVisit(this, scope);
  }
}
